@@ -11,18 +11,21 @@ import argparse
 import locale
 import collections
 
+# pelican.log has to be the first pelican module to be loaded
+# because logging.setLoggerClass has to be called before logging.getLogger
+from pelican.log import init
+
 from pelican import signals
 
 from pelican.generators import (ArticlesGenerator, PagesGenerator,
                                 StaticGenerator, SourceFileGenerator,
                                 TemplatePagesGenerator)
-from pelican.log import init
 from pelican.readers import Readers
 from pelican.settings import read_settings
 from pelican.utils import clean_output_dir, folder_watcher, file_watcher
 from pelican.writers import Writer
 
-__version__ = "3.3.0"
+__version__ = "3.4.0"
 
 DEFAULT_CONFIG_NAME = 'pelicanconf.py'
 
@@ -31,6 +34,7 @@ logger = logging.getLogger(__name__)
 
 
 class Pelican(object):
+
     def __init__(self, settings):
         """
         Pelican initialisation, performs some checks on the environment before
@@ -59,17 +63,20 @@ class Pelican(object):
 
     def init_plugins(self):
         self.plugins = []
-        logger.debug('Temporarily adding PLUGIN_PATH to system path')
+        logger.debug('Temporarily adding PLUGIN_PATHS to system path')
         _sys_path = sys.path[:]
-        sys.path.insert(0, self.settings['PLUGIN_PATH'])
+        for pluginpath in self.settings['PLUGIN_PATHS']:
+            sys.path.insert(0, pluginpath)
         for plugin in self.settings['PLUGINS']:
             # if it's a string, then import it
             if isinstance(plugin, six.string_types):
                 logger.debug("Loading plugin `{0}`".format(plugin))
                 try:
-                    plugin = __import__(plugin, globals(), locals(), str('module'))
+                    plugin = __import__(plugin, globals(), locals(),
+                                        str('module'))
                 except ImportError as e:
-                    logger.error("Can't find plugin `{0}`: {1}".format(plugin, e))
+                    logger.error(
+                        "Can't find plugin `{0}`: {1}".format(plugin, e))
                     continue
 
             logger.debug("Registering plugin `{0}`".format(plugin.__name__))
@@ -82,8 +89,8 @@ class Pelican(object):
 
         if self.settings.get('CLEAN_URLS', False):
             logger.warning('Found deprecated `CLEAN_URLS` in settings.'
-                        ' Modifying the following settings for the'
-                        ' same behaviour.')
+                           ' Modifying the following settings for the'
+                           ' same behaviour.')
 
             self.settings['ARTICLE_URL'] = '{slug}/'
             self.settings['ARTICLE_LANG_URL'] = '{slug}-{lang}/'
@@ -96,8 +103,8 @@ class Pelican(object):
 
         if self.settings.get('ARTICLE_PERMALINK_STRUCTURE', False):
             logger.warning('Found deprecated `ARTICLE_PERMALINK_STRUCTURE` in'
-                        ' settings.  Modifying the following settings for'
-                        ' the same behaviour.')
+                           ' settings.  Modifying the following settings for'
+                           ' the same behaviour.')
 
             structure = self.settings['ARTICLE_PERMALINK_STRUCTURE']
 
@@ -111,41 +118,26 @@ class Pelican(object):
             structure = re.sub('^/', '', structure)
 
             for setting in ('ARTICLE_URL', 'ARTICLE_LANG_URL', 'PAGE_URL',
-                            'PAGE_LANG_URL', 'ARTICLE_SAVE_AS',
-                            'ARTICLE_LANG_SAVE_AS', 'PAGE_SAVE_AS',
-                            'PAGE_LANG_SAVE_AS'):
+                            'PAGE_LANG_URL', 'DRAFT_URL', 'DRAFT_LANG_URL',
+                            'ARTICLE_SAVE_AS', 'ARTICLE_LANG_SAVE_AS',
+                            'DRAFT_SAVE_AS', 'DRAFT_LANG_SAVE_AS',
+                            'PAGE_SAVE_AS', 'PAGE_LANG_SAVE_AS'):
                 self.settings[setting] = os.path.join(structure,
                                                       self.settings[setting])
                 logger.warning("%s = '%s'" % (setting, self.settings[setting]))
 
-        if self.settings.get('FEED', False):
-            logger.warning('Found deprecated `FEED` in settings. Modify FEED'
-            ' to FEED_ATOM in your settings and theme for the same behavior.'
-            ' Temporarily setting FEED_ATOM for backwards compatibility.')
-            self.settings['FEED_ATOM'] = self.settings['FEED']
-
-        if self.settings.get('TAG_FEED', False):
-            logger.warning('Found deprecated `TAG_FEED` in settings. Modify '
-            ' TAG_FEED to TAG_FEED_ATOM in your settings and theme for the '
-            'same behavior. Temporarily setting TAG_FEED_ATOM for backwards '
-            'compatibility.')
-            self.settings['TAG_FEED_ATOM'] = self.settings['TAG_FEED']
-
-        if self.settings.get('CATEGORY_FEED', False):
-            logger.warning('Found deprecated `CATEGORY_FEED` in settings. '
-            'Modify CATEGORY_FEED to CATEGORY_FEED_ATOM in your settings and '
-            'theme for the same behavior. Temporarily setting '
-            'CATEGORY_FEED_ATOM for backwards compatibility.')
-            self.settings['CATEGORY_FEED_ATOM'] =\
-                    self.settings['CATEGORY_FEED']
-
-        if self.settings.get('TRANSLATION_FEED', False):
-            logger.warning('Found deprecated `TRANSLATION_FEED` in settings. '
-            'Modify TRANSLATION_FEED to TRANSLATION_FEED_ATOM in your '
-            'settings and theme for the same behavior. Temporarily setting '
-            'TRANSLATION_FEED_ATOM for backwards compatibility.')
-            self.settings['TRANSLATION_FEED_ATOM'] =\
-                    self.settings['TRANSLATION_FEED']
+        for new, old in [('FEED', 'FEED_ATOM'), ('TAG_FEED', 'TAG_FEED_ATOM'),
+                         ('CATEGORY_FEED', 'CATEGORY_FEED_ATOM'),
+                         ('TRANSLATION_FEED', 'TRANSLATION_FEED_ATOM')]:
+            if self.settings.get(new, False):
+                logger.warning(
+                    'Found deprecated `%(new)s` in settings. Modify %(new)s '
+                    'to %(old)s in your settings and theme for the same '
+                    'behavior. Temporarily setting %(old)s for backwards '
+                    'compatibility.',
+                    {'new': new, 'old': old}
+                )
+                self.settings[old] = self.settings[new]
 
     def run(self):
         """Run the generators and return"""
@@ -182,11 +174,16 @@ class Pelican(object):
 
         signals.finalized.send(self)
 
-        articles_generator = next(g for g in generators if isinstance(g, ArticlesGenerator))
-        pages_generator = next(g for g in generators if isinstance(g, PagesGenerator))
+        articles_generator = next(g for g in generators
+                                  if isinstance(g, ArticlesGenerator))
+        pages_generator = next(g for g in generators
+                               if isinstance(g, PagesGenerator))
 
-        print('Done: Processed {} articles and {} pages in {:.2f} seconds.'.format(
+        print('Done: Processed {} article(s), {} draft(s) and {} page(s) in ' \
+              '{:.2f} seconds.'.format(
             len(articles_generator.articles) + len(articles_generator.translations),
+            len(articles_generator.drafts) + \
+            len(articles_generator.drafts_translations),
             len(pages_generator.pages) + len(pages_generator.translations),
             time.time() - start_time))
 
@@ -212,55 +209,84 @@ class Pelican(object):
         return generators
 
     def get_writer(self):
-        return Writer(self.output_path, settings=self.settings)
+        writers = [ w for (_, w) in signals.get_writer.send(self)
+                    if isinstance(w, type) ]
+        writers_found = len(writers)
+        if writers_found == 0:
+            return Writer(self.output_path, settings=self.settings)
+        else:
+            writer = writers[0]
+            if writers_found == 1:
+                logger.debug('Found writer: {}'.format(writer))
+            else:
+                logger.warning(
+                    '{} writers found, using only first one: {}'.format(
+                    writers_found, writer))
+            return writer(self.output_path, settings=self.settings)
 
 
 def parse_arguments():
-    parser = argparse.ArgumentParser(description="""A tool to generate a
-    static blog, with restructured text input files.""",
-        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+    parser = argparse.ArgumentParser(
+        description="""A tool to generate a static blog,
+        with restructured text input files.""",
+        formatter_class=argparse.ArgumentDefaultsHelpFormatter
+    )
 
     parser.add_argument(dest='path', nargs='?',
-        help='Path where to find the content files.',
-        default=None)
+                        help='Path where to find the content files.',
+                        default=None)
 
     parser.add_argument('-t', '--theme-path', dest='theme',
-        help='Path where to find the theme templates. If not specified, it'
-             'will use the default one included with pelican.')
+                        help='Path where to find the theme templates. If not '
+                        'specified, it will use the default one included with '
+                        'pelican.')
 
     parser.add_argument('-o', '--output', dest='output',
-        help='Where to output the generated files. If not specified, a '
-             'directory will be created, named "output" in the current path.')
+                        help='Where to output the generated files. If not '
+                        'specified, a directory will be created, named '
+                        '"output" in the current path.')
 
     parser.add_argument('-s', '--settings', dest='settings',
-        help='The settings of the application, this is automatically set to '
-        '{0} if a file exists with this name.'.format(DEFAULT_CONFIG_NAME))
+                        help='The settings of the application, this is '
+                        'automatically set to {0} if a file exists with this '
+                        'name.'.format(DEFAULT_CONFIG_NAME))
 
     parser.add_argument('-d', '--delete-output-directory',
-        dest='delete_outputdir',
-        action='store_true',
-        default=None,
-        help='Delete the output directory.')
+                        dest='delete_outputdir', action='store_true',
+                        default=None, help='Delete the output directory.')
 
     parser.add_argument('-v', '--verbose', action='store_const',
-        const=logging.INFO, dest='verbosity',
-        help='Show all messages.')
+                        const=logging.INFO, dest='verbosity',
+                        help='Show all messages.')
 
     parser.add_argument('-q', '--quiet', action='store_const',
-        const=logging.CRITICAL, dest='verbosity',
-        help='Show only critical errors.')
+                        const=logging.CRITICAL, dest='verbosity',
+                        help='Show only critical errors.')
 
     parser.add_argument('-D', '--debug', action='store_const',
-        const=logging.DEBUG, dest='verbosity',
-        help='Show all message, including debug messages.')
+                        const=logging.DEBUG, dest='verbosity',
+                        help='Show all messages, including debug messages.')
 
     parser.add_argument('--version', action='version', version=__version__,
-        help='Print the pelican version and exit.')
+                        help='Print the pelican version and exit.')
 
     parser.add_argument('-r', '--autoreload', dest='autoreload',
-        action='store_true',
-        help="Relaunch pelican each time a modification occurs"
-                             " on the content files.")
+                        action='store_true',
+                        help='Relaunch pelican each time a modification occurs'
+                        ' on the content files.')
+
+    parser.add_argument('--cache-path', dest='cache_path',
+                        help=('Directory in which to store cache files. '
+                              'If not specified, defaults to "cache".'))
+
+    parser.add_argument('--ignore-cache', action='store_true',
+                        dest='ignore_cache', help='Ignore content cache '
+                        'from previous runs by not loading cache files.')
+
+    parser.add_argument('-w', '--write-selected', type=str,
+                        dest='selected_paths', default=None,
+                        help='Comma separated list of selected paths to write')
+
     return parser.parse_args()
 
 
@@ -270,12 +296,18 @@ def get_config(args):
         config['PATH'] = os.path.abspath(os.path.expanduser(args.path))
     if args.output:
         config['OUTPUT_PATH'] = \
-                os.path.abspath(os.path.expanduser(args.output))
+            os.path.abspath(os.path.expanduser(args.output))
     if args.theme:
         abstheme = os.path.abspath(os.path.expanduser(args.theme))
         config['THEME'] = abstheme if os.path.exists(abstheme) else args.theme
     if args.delete_outputdir is not None:
         config['DELETE_OUTPUT_DIRECTORY'] = args.delete_outputdir
+    if args.ignore_cache:
+        config['LOAD_CONTENT_CACHE'] = False
+    if args.cache_path:
+        config['CACHE_PATH'] = args.cache_path
+    if args.selected_paths:
+        config['WRITE_SELECTED'] = args.selected_paths.split(',')
 
     # argparse returns bytes in Py2. There is no definite answer as to which
     # encoding argparse (or sys.argv) uses.
@@ -320,11 +352,18 @@ def main():
                                         pelican.ignore_files),
                 'settings': file_watcher(args.settings)}
 
+    for static_path in settings.get("STATIC_PATHS", []):
+        watchers[static_path] = folder_watcher(static_path, [''], pelican.ignore_files)
+
     try:
         if args.autoreload:
             print('  --- AutoReload Mode: Monitoring `content`, `theme` and'
                   ' `settings` for changes. ---')
 
+            def _ignore_cache(pelican_obj):
+                if pelican_obj.settings['AUTORELOAD_IGNORE_CACHE']:
+                    pelican_obj.settings['LOAD_CONTENT_CACHE'] = False
+
             while True:
                 try:
                     # Check source dir for changed files ending with the given
@@ -333,21 +372,28 @@ def main():
                     # have changed, no matter what extension the filenames
                     # have.
                     modified = {k: next(v) for k, v in watchers.items()}
+                    original_load_cache = settings['LOAD_CONTENT_CACHE']
 
                     if modified['settings']:
-                        pelican, settings  = get_instance(args)
+                        pelican, settings = get_instance(args)
+                        original_load_cache = settings['LOAD_CONTENT_CACHE']
+                        print(pelican.settings['AUTORELOAD_IGNORE_CACHE'])
+                        _ignore_cache(pelican)
 
                     if any(modified.values()):
                         print('\n-> Modified: {}. re-generating...'.format(
-                                ', '.join(k for k, v in modified.items() if v)))
+                            ', '.join(k for k, v in modified.items() if v)))
 
                         if modified['content'] is None:
                             logger.warning('No valid files found in content.')
 
                         if modified['theme'] is None:
-                            logger.warning('Empty theme folder. Using `basic` theme.')
+                            logger.warning('Empty theme folder. Using `basic` '
+                                           'theme.')
 
                         pelican.run()
+                        # restore original caching policy
+                        pelican.settings['LOAD_CONTENT_CACHE'] = original_load_cache
 
                 except KeyboardInterrupt:
                     logger.warning("Keyboard interrupt, quitting.")
@@ -358,7 +404,7 @@ def main():
                         logger.critical(e.args)
                         raise
                     logger.warning(
-                            'Caught exception "{0}". Reloading.'.format(e))
+                        'Caught exception "{0}". Reloading.'.format(e))
 
                 finally:
                     time.sleep(.5)  # sleep to avoid cpu load
@@ -381,7 +427,7 @@ def main():
 
         logger.critical(msg)
 
-        if (args.verbosity == logging.DEBUG):
+        if args.verbosity == logging.DEBUG:
             raise
         else:
             sys.exit(getattr(e, 'exitcode', 1))
