@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals, print_function
 import six
+from six.moves.urllib.parse import (unquote, urlparse, urlunparse)
 
 import copy
 import locale
@@ -10,14 +11,12 @@ import os
 import re
 import sys
 
-from datetime import datetime
-
 
 from pelican import signals
 from pelican.settings import DEFAULT_CONFIG
 from pelican.utils import (slugify, truncate_html_words, memoized, strftime,
                            python_2_unicode_compatible, deprecated_attribute,
-                           path_to_url)
+                           path_to_url, SafeDatetime)
 
 # Import these so that they're avalaible when you import from pelican.contents.
 from pelican.urlwrappers import (URLWrapper, Author, Category, Tag)  # NOQA
@@ -68,11 +67,17 @@ class Content(object):
         #default template if it's not defined in page
         self.template = self._get_template()
 
-        # default author to the one in settings if not defined
+        # First, read the authors from "authors", if not, fallback to "author"
+        # and if not use the settings defined one, if any.
         if not hasattr(self, 'author'):
-            if 'AUTHOR' in settings:
+            if hasattr(self, 'authors'):
+                self.author = self.authors[0]
+            elif 'AUTHOR' in settings:
                 self.author = Author(settings['AUTHOR'], settings)
 
+        if not hasattr(self, 'authors') and hasattr(self, 'author'):
+            self.authors = [self.author]
+
         # XXX Split all the following code into pieces, there is too much here.
 
         # manage languages
@@ -84,9 +89,15 @@ class Content(object):
 
             self.in_default_lang = (self.lang == default_lang)
 
-        # create the slug if not existing, from the title
-        if not hasattr(self, 'slug') and hasattr(self, 'title'):
-            self.slug = slugify(self.title,
+        # create the slug if not existing, generate slug according to 
+        # setting of SLUG_ATTRIBUTE
+        if not hasattr(self, 'slug'):
+            if settings['SLUGIFY_SOURCE'] == 'title' and hasattr(self, 'title'):
+                self.slug = slugify(self.title,
+                                settings.get('SLUG_SUBSTITUTIONS', ()))
+            elif settings['SLUGIFY_SOURCE'] == 'basename' and source_path != None:
+                basename = os.path.basename(os.path.splitext(source_path)[0])
+                self.slug = slugify(basename,
                                 settings.get('SLUG_SUBSTITUTIONS', ()))
 
         self.source_path = source_path
@@ -108,12 +119,14 @@ class Content(object):
 
         if hasattr(self, 'date'):
             self.locale_date = strftime(self.date, self.date_format)
+        if hasattr(self, 'modified'):
+            self.locale_modified = strftime(self.modified, self.date_format)
 
         # manage status
         if not hasattr(self, 'status'):
             self.status = settings['DEFAULT_STATUS']
             if not settings['WITH_FUTURE_DATES']:
-                if hasattr(self, 'date') and self.date > datetime.now():
+                if hasattr(self, 'date') and self.date > SafeDatetime.now():
                     self.status = 'draft'
 
         # store the summary metadata if it is set
@@ -147,7 +160,7 @@ class Content(object):
             'path': path_to_url(path),
             'slug': getattr(self, 'slug', ''),
             'lang': getattr(self, 'lang', 'en'),
-            'date': getattr(self, 'date', datetime.now()),
+            'date': getattr(self, 'date', SafeDatetime.now()),
             'author': slugify(
                 getattr(self, 'author', ''),
                 slug_substitutions
@@ -184,8 +197,8 @@ class Content(object):
 
         instrasite_link_regex = self.settings['INTRASITE_LINK_REGEX']
         regex = r"""
-            (?P<markup><\s*[^\>]*  # match tag with src and href attr
-                (?:href|src)\s*=)
+            (?P<markup><\s*[^\>]*  # match tag with all url-value attributes
+                (?:href|src|poster|data|cite|formaction|action)\s*=)
 
             (?P<quote>["\'])      # require value to be quoted
             (?P<path>{0}(?P<value>.*?))  # the url value
@@ -194,34 +207,44 @@ class Content(object):
 
         def replacer(m):
             what = m.group('what')
-            value = m.group('value')
+            value = urlparse(m.group('value'))
+            path = value.path
             origin = m.group('path')
 
-            # we support only filename for now. the plan is to support
-            # categories, tags, etc. in the future, but let's keep things
-            # simple for now.
-
             # XXX Put this in a different location.
             if what == 'filename':
-                if value.startswith('/'):
-                    value = value[1:]
+                if path.startswith('/'):
+                    path = path[1:]
                 else:
                     # relative to the source path of this content
-                    value = self.get_relative_source_path(
-                        os.path.join(self.relative_dir, value)
+                    path = self.get_relative_source_path(
+                        os.path.join(self.relative_dir, path)
                     )
 
-                if value in self._context['filenames']:
+                if path not in self._context['filenames']:
+                    unquoted_path = path.replace('%20', ' ')
+
+                    if unquoted_path in self._context['filenames']:
+                        path = unquoted_path
+
+                if path in self._context['filenames']:
                     origin = '/'.join((siteurl,
-                             self._context['filenames'][value].url))
-                    origin = origin.replace('\\', '/')  # Fow windows paths.
+                             self._context['filenames'][path].url))
+                    origin = origin.replace('\\', '/')  # for Windows paths.
                 else:
-                    logger.warning("Unable to find {fn}, skipping url"
-                                   " replacement".format(fn=value))
+                    logger.warning(("Unable to find {fn}, skipping url"
+                                    " replacement".format(fn=value),
+                                    "Other resources were not found"
+                                    " and their urls not replaced"))
             elif what == 'category':
-                origin = Category(value, self.settings).url
+                origin = Category(path, self.settings).url
             elif what == 'tag':
-                origin = Tag(value, self.settings).url
+                origin = Tag(path, self.settings).url
+
+            # keep all other parts, such as query, fragment, etc.
+            parts = list(value)
+            parts[2] = origin
+            origin = urlunparse(parts)
 
             return ''.join((m.group('markup'), m.group('quote'), origin,
                             m.group('quote')))
@@ -306,6 +329,11 @@ class Article(Page):
     default_template = 'article'
 
 
+class Draft(Page):
+    mandatory_properties = ('title', 'category')
+    default_template = 'article'
+
+
 class Quote(Page):
     base_properties = ('author', 'date')
 
