@@ -2,10 +2,13 @@
 from __future__ import unicode_literals, print_function
 
 import os
+import six
 import math
 import random
 import logging
 import shutil
+import fnmatch
+import calendar
 
 from codecs import open
 from collections import defaultdict
@@ -16,9 +19,10 @@ from operator import attrgetter, itemgetter
 from jinja2 import (Environment, FileSystemLoader, PrefixLoader, ChoiceLoader,
                     BaseLoader, TemplateNotFound)
 
-from pelican.contents import Article, Page, Static, is_valid_content
+from pelican.contents import Article, Draft, Page, Static, is_valid_content
 from pelican.readers import Readers
-from pelican.utils import copy, process_translations, mkdir_p, DateFormatter
+from pelican.utils import (copy, process_translations, mkdir_p, DateFormatter,
+                           FileStampDataCacher)
 from pelican import signals
 
 
@@ -28,7 +32,8 @@ logger = logging.getLogger(__name__)
 class Generator(object):
     """Baseclass generator"""
 
-    def __init__(self, context, settings, path, theme, output_path, **kwargs):
+    def __init__(self, context, settings, path, theme, output_path,
+                 readers_cache_name='', **kwargs):
         self.context = context
         self.settings = settings
         self.path = path
@@ -38,7 +43,7 @@ class Generator(object):
         for arg, value in kwargs.items():
             setattr(self, arg, value)
 
-        self.readers = Readers(self.settings)
+        self.readers = Readers(self.settings, readers_cache_name)
 
         # templates cache
         self._templates = {}
@@ -96,33 +101,42 @@ class Generator(object):
         if extensions is None:
             extensions = tuple(self.readers.extensions)
         basename = os.path.basename(path)
+
+        #check IGNORE_FILES
+        ignores = self.settings['IGNORE_FILES']
+        if any(fnmatch.fnmatch(basename, ignore) for ignore in ignores):
+            return False
+
         if extensions is False or basename.endswith(extensions):
             return True
         return False
 
-    def get_files(self, path, exclude=[], extensions=None):
+    def get_files(self, paths, exclude=[], extensions=None):
         """Return a list of files to use, based on rules
 
-        :param path: the path to search (relative to self.path)
+        :param paths: the list pf paths to search (relative to self.path)
         :param exclude: the list of path to exclude
         :param extensions: the list of allowed extensions (if False, all
             extensions are allowed)
         """
+        if isinstance(paths, six.string_types):
+            paths = [paths] # backward compatibility for older generators
         files = []
-        root = os.path.join(self.path, path)
+        for path in paths:
+            root = os.path.join(self.path, path)
 
-        if os.path.isdir(root):
-            for dirpath, dirs, temp_files in os.walk(root, followlinks=True):
-                for e in exclude:
-                    if e in dirs:
-                        dirs.remove(e)
-                reldir = os.path.relpath(dirpath, self.path)
-                for f in temp_files:
-                    fp = os.path.join(reldir, f)
-                    if self._include_path(fp, extensions):
-                        files.append(fp)
-        elif os.path.exists(root) and self._include_path(path, extensions):
-            files.append(path)  # can't walk non-directories
+            if os.path.isdir(root):
+                for dirpath, dirs, temp_files in os.walk(root, followlinks=True):
+                    for e in exclude:
+                        if e in dirs:
+                            dirs.remove(e)
+                    reldir = os.path.relpath(dirpath, self.path)
+                    for f in temp_files:
+                        fp = os.path.join(reldir, f)
+                        if self._include_path(fp, extensions):
+                            files.append(fp)
+            elif os.path.exists(root) and self._include_path(path, extensions):
+                files.append(path)  # can't walk non-directories
         return files
 
     def add_source_path(self, content):
@@ -140,6 +154,35 @@ class Generator(object):
             self.context[item] = value
 
 
+class CachingGenerator(Generator, FileStampDataCacher):
+    '''Subclass of Generator and FileStampDataCacher classes
+
+    enables content caching, either at the generator or reader level
+    '''
+
+    def __init__(self, *args, **kwargs):
+        '''Initialize the generator, then set up caching
+
+        note the multiple inheritance structure
+        '''
+        cls_name = self.__class__.__name__
+        Generator.__init__(self, *args,
+                           readers_cache_name=(cls_name + '-Readers'),
+                           **kwargs)
+
+        cache_this_level = self.settings['CONTENT_CACHING_LAYER'] == 'generator'
+        caching_policy = cache_this_level and self.settings['CACHE_CONTENT']
+        load_policy = cache_this_level and self.settings['LOAD_CONTENT_CACHE']
+        FileStampDataCacher.__init__(self, self.settings, cls_name,
+                                     caching_policy, load_policy
+                                     )
+
+    def _get_file_stamp(self, filename):
+        '''Get filestamp for path relative to generator.path'''
+        filename = os.path.join(self.path, filename)
+        return super(CachingGenerator, self)._get_file_stamp(filename)
+
+
 class _FileLoader(BaseLoader):
 
     def __init__(self, path, basedir):
@@ -170,7 +213,7 @@ class TemplatePagesGenerator(Generator):
                 del self.env.loader.loaders[0]
 
 
-class ArticlesGenerator(Generator):
+class ArticlesGenerator(CachingGenerator):
     """Generate blog articles"""
 
     def __init__(self, *args, **kwargs):
@@ -182,7 +225,8 @@ class ArticlesGenerator(Generator):
         self.categories = defaultdict(list)
         self.related_posts = []
         self.authors = defaultdict(list)
-        self.drafts = []
+        self.drafts = [] # only drafts in default language
+        self.drafts_translations = []
         super(ArticlesGenerator, self).__init__(*args, **kwargs)
         signals.article_generator_init.send(self)
 
@@ -225,6 +269,18 @@ class ArticlesGenerator(Generator):
                                   self.settings['CATEGORY_FEED_RSS']
                                   % cat.slug, feed_type='rss')
 
+        for auth, arts in self.authors:
+            arts.sort(key=attrgetter('date'), reverse=True)
+            if self.settings.get('AUTHOR_FEED_ATOM'):
+                writer.write_feed(arts, self.context,
+                                  self.settings['AUTHOR_FEED_ATOM']
+                                  % auth.slug)
+
+            if self.settings.get('AUTHOR_FEED_RSS'):
+                writer.write_feed(arts, self.context,
+                                  self.settings['AUTHOR_FEED_RSS']
+                                  % auth.slug, feed_type='rss')
+
         if (self.settings.get('TAG_FEED_ATOM')
                 or self.settings.get('TAG_FEED_RSS')):
             for tag, arts in self.tags.items():
@@ -248,19 +304,22 @@ class ArticlesGenerator(Generator):
             for lang, items in translations_feeds.items():
                 items.sort(key=attrgetter('date'), reverse=True)
                 if self.settings.get('TRANSLATION_FEED_ATOM'):
-                    writer.write_feed(items, self.context,
-                            self.settings['TRANSLATION_FEED_ATOM'] % lang)
+                    writer.write_feed(
+                        items, self.context,
+                        self.settings['TRANSLATION_FEED_ATOM'] % lang)
                 if self.settings.get('TRANSLATION_FEED_RSS'):
-                    writer.write_feed(items, self.context,
-                            self.settings['TRANSLATION_FEED_RSS'] % lang,
-                            feed_type='rss')
+                    writer.write_feed(
+                        items, self.context,
+                        self.settings['TRANSLATION_FEED_RSS'] % lang,
+                        feed_type='rss')
 
     def generate_articles(self, write):
         """Generate the articles."""
         for article in chain(self.translations, self.articles):
+            signals.article_generator_write_article.send(self, content=article)
             write(article.save_as, self.get_template(article.template),
-                self.context, article=article, category=article.category,
-                override_output=hasattr(article, 'override_save_as'))
+                  self.context, article=article, category=article.category,
+                  override_output=hasattr(article, 'override_save_as'))
 
     def generate_period_archives(self, write):
         """Generate per-year, per-month, and per-day archives."""
@@ -269,6 +328,18 @@ class ArticlesGenerator(Generator):
         except Exception:
             template = self.get_template('archives')
 
+        period_save_as = {
+            'year': self.settings['YEAR_ARCHIVE_SAVE_AS'],
+            'month': self.settings['MONTH_ARCHIVE_SAVE_AS'],
+            'day': self.settings['DAY_ARCHIVE_SAVE_AS'],
+        }
+
+        period_date_key = {
+            'year': attrgetter('date.year'),
+            'month': attrgetter('date.year', 'date.month'),
+            'day': attrgetter('date.year', 'date.month', 'date.day')
+        }
+
         def _generate_period_archives(dates, key, save_as_fmt):
             """Generate period archives from `dates`, grouped by
             `key` and written to `save_as`.
@@ -281,21 +352,25 @@ class ArticlesGenerator(Generator):
                 # period archive dates
                 date = archive[0].date
                 save_as = save_as_fmt.format(date=date)
-                write(save_as, template, self.context,
+                context = self.context.copy()
+
+                if key == period_date_key['year']:
+                    context["period"] = (_period,)
+                else:
+                    month_name = calendar.month_name[_period[1]]
+                    if not six.PY3:
+                        month_name = month_name.decode('utf-8')
+                    if key == period_date_key['month']:
+                        context["period"] = (_period[0],
+                                             month_name)
+                    else:
+                        context["period"] = (_period[0],
+                                             month_name,
+                                             _period[2])
+
+                write(save_as, template, context,
                       dates=archive, blog=True)
 
-        period_save_as = {
-                'year' : self.settings['YEAR_ARCHIVE_SAVE_AS'],
-                'month': self.settings['MONTH_ARCHIVE_SAVE_AS'],
-                'day'  : self.settings['DAY_ARCHIVE_SAVE_AS'],
-                }
-
-        period_date_key = {
-                'year' : attrgetter('date.year'),
-                'month': attrgetter('date.year', 'date.month'),
-                'day'  : attrgetter('date.year', 'date.month', 'date.day')
-                }
-
         for period in 'year', 'month', 'day':
             save_as = period_save_as[period]
             if save_as:
@@ -310,13 +385,13 @@ class ArticlesGenerator(Generator):
             if template in PAGINATED_TEMPLATES:
                 paginated = {'articles': self.articles, 'dates': self.dates}
             save_as = self.settings.get("%s_SAVE_AS" % template.upper(),
-                                                        '%s.html' % template)
+                                        '%s.html' % template)
             if not save_as:
                 continue
 
             write(save_as, self.get_template(template),
-                self.context, blog=True, paginated=paginated,
-                page_name=os.path.splitext(save_as)[0])
+                  self.context, blog=True, paginated=paginated,
+                  page_name=os.path.splitext(save_as)[0])
 
     def generate_tags(self, write):
         """Generate Tags pages."""
@@ -325,9 +400,9 @@ class ArticlesGenerator(Generator):
             articles.sort(key=attrgetter('date'), reverse=True)
             dates = [article for article in self.dates if article in articles]
             write(tag.save_as, tag_template, self.context, tag=tag,
-                articles=articles, dates=dates,
-                paginated={'articles': articles, 'dates': dates},
-                page_name=tag.page_name, all_articles=self.articles)
+                  articles=articles, dates=dates,
+                  paginated={'articles': articles, 'dates': dates},
+                  page_name=tag.page_name, all_articles=self.articles)
 
     def generate_categories(self, write):
         """Generate category pages."""
@@ -336,9 +411,9 @@ class ArticlesGenerator(Generator):
             articles.sort(key=attrgetter('date'), reverse=True)
             dates = [article for article in self.dates if article in articles]
             write(cat.save_as, category_template, self.context,
-                category=cat, articles=articles, dates=dates,
-                paginated={'articles': articles, 'dates': dates},
-                page_name=cat.page_name, all_articles=self.articles)
+                  category=cat, articles=articles, dates=dates,
+                  paginated={'articles': articles, 'dates': dates},
+                  page_name=cat.page_name, all_articles=self.articles)
 
     def generate_authors(self, write):
         """Generate Author pages."""
@@ -347,16 +422,16 @@ class ArticlesGenerator(Generator):
             articles.sort(key=attrgetter('date'), reverse=True)
             dates = [article for article in self.dates if article in articles]
             write(aut.save_as, author_template, self.context,
-                author=aut, articles=articles, dates=dates,
-                paginated={'articles': articles, 'dates': dates},
-                page_name=aut.page_name, all_articles=self.articles)
+                  author=aut, articles=articles, dates=dates,
+                  paginated={'articles': articles, 'dates': dates},
+                  page_name=aut.page_name, all_articles=self.articles)
 
     def generate_drafts(self, write):
         """Generate drafts pages."""
-        for article in self.drafts:
-            write(os.path.join('drafts', '%s.html' % article.slug),
-                self.get_template(article.template), self.context,
-                article=article, category=article.category,
+        for draft in chain(self.drafts_translations, self.drafts):
+            write(draft.save_as, self.get_template(draft.template),
+                self.context, article=draft, category=draft.category,
+                override_output=hasattr(draft, 'override_save_as'),
                 all_articles=self.articles)
 
     def generate_pages(self, writer):
@@ -380,36 +455,52 @@ class ArticlesGenerator(Generator):
         """Add the articles into the shared context"""
 
         all_articles = []
+        all_drafts = []
         for f in self.get_files(
-                self.settings['ARTICLE_DIR'],
+                self.settings['ARTICLE_PATHS'],
                 exclude=self.settings['ARTICLE_EXCLUDES']):
-            try:
-                article = self.readers.read_file(
-                    base_path=self.path, path=f, content_class=Article,
+            article = self.get_cached_data(f, None)
+            if article is None:
+                try:
+                    article = self.readers.read_file(
+                        base_path=self.path, path=f, content_class=Article,
+                        context=self.context,
+                        preread_signal=signals.article_generator_preread,
+                        preread_sender=self,
+                        context_signal=signals.article_generator_context,
+                        context_sender=self)
+                except Exception as e:
+                    logger.warning('Could not process {}\n{}'.format(f, e))
+                    continue
+
+                if not is_valid_content(article, f):
+                    continue
+
+                self.cache_data(f, article)
+
+            self.add_source_path(article)
+
+            if article.status.lower() == "published":
+                all_articles.append(article)
+            elif article.status.lower() == "draft":
+                draft = self.readers.read_file(
+                    base_path=self.path, path=f, content_class=Draft,
                     context=self.context,
                     preread_signal=signals.article_generator_preread,
                     preread_sender=self,
                     context_signal=signals.article_generator_context,
                     context_sender=self)
-            except Exception as e:
-                logger.warning('Could not process {}\n{}'.format(f, e))
-                continue
-
-            if not is_valid_content(article, f):
-                continue
-
-            self.add_source_path(article)
-
-            if article.status == "published":
-                all_articles.append(article)
-            elif article.status == "draft":
-                self.drafts.append(article)
+                all_drafts.append(draft)
             else:
                 logger.warning("Unknown status %s for file %s, skipping it." %
                                (repr(article.status),
                                 repr(f)))
 
         self.articles, self.translations = process_translations(all_articles)
+        self.drafts, self.drafts_translations = \
+            process_translations(all_drafts)
+
+        signals.article_generator_pretaxonomy.send(self)
 
         for article in self.articles:
             # only main articles are listed in categories and tags
@@ -419,15 +510,14 @@ class ArticlesGenerator(Generator):
                 for tag in article.tags:
                     self.tags[tag].append(article)
             # ignore blank authors as well as undefined
-            if hasattr(article, 'author') and article.author.name != '':
-                self.authors[article.author].append(article)
-
-
+            for author in getattr(article, 'authors', []):
+                if author.name != '':
+                    self.authors[author].append(article)
         # sort the articles by date
         self.articles.sort(key=attrgetter('date'), reverse=True)
         self.dates = list(self.articles)
         self.dates.sort(key=attrgetter('date'),
-                reverse=self.context['NEWEST_FIRST_ARCHIVES'])
+                        reverse=self.context['NEWEST_FIRST_ARCHIVES'])
 
         # create tag cloud
         tag_cloud = defaultdict(int)
@@ -460,22 +550,24 @@ class ArticlesGenerator(Generator):
         # order the categories per name
         self.categories = list(self.categories.items())
         self.categories.sort(
-                reverse=self.settings['REVERSE_CATEGORY_ORDER'])
+            reverse=self.settings['REVERSE_CATEGORY_ORDER'])
 
         self.authors = list(self.authors.items())
         self.authors.sort()
 
         self._update_context(('articles', 'dates', 'tags', 'categories',
                               'tag_cloud', 'authors', 'related_posts'))
-
+        self.save_cache()
+        self.readers.save_cache()
         signals.article_generator_finalized.send(self)
 
     def generate_output(self, writer):
         self.generate_feeds(writer)
         self.generate_pages(writer)
+        signals.article_writer_finalized.send(self, writer=writer)
 
 
-class PagesGenerator(Generator):
+class PagesGenerator(CachingGenerator):
     """Generate pages"""
 
     def __init__(self, *args, **kwargs):
@@ -489,22 +581,26 @@ class PagesGenerator(Generator):
         all_pages = []
         hidden_pages = []
         for f in self.get_files(
-                self.settings['PAGE_DIR'],
+                self.settings['PAGE_PATHS'],
                 exclude=self.settings['PAGE_EXCLUDES']):
-            try:
-                page = self.readers.read_file(
-                    base_path=self.path, path=f, content_class=Page,
-                    context=self.context,
-                    preread_signal=signals.page_generator_preread,
-                    preread_sender=self,
-                    context_signal=signals.page_generator_context,
-                    context_sender=self)
-            except Exception as e:
-                logger.warning('Could not process {}\n{}'.format(f, e))
-                continue
+            page = self.get_cached_data(f, None)
+            if page is None:
+                try:
+                    page = self.readers.read_file(
+                        base_path=self.path, path=f, content_class=Page,
+                        context=self.context,
+                        preread_signal=signals.page_generator_preread,
+                        preread_sender=self,
+                        context_signal=signals.page_generator_context,
+                        context_sender=self)
+                except Exception as e:
+                    logger.warning('Could not process {}\n{}'.format(f, e))
+                    continue
 
-            if not is_valid_content(page, f):
-                continue
+                if not is_valid_content(page, f):
+                    continue
+
+                self.cache_data(f, page)
 
             self.add_source_path(page)
 
@@ -519,50 +615,59 @@ class PagesGenerator(Generator):
 
         self.pages, self.translations = process_translations(all_pages)
         self.hidden_pages, self.hidden_translations = (
-                process_translations(hidden_pages))
+            process_translations(hidden_pages))
 
         self._update_context(('pages', ))
         self.context['PAGES'] = self.pages
 
+        self.save_cache()
+        self.readers.save_cache()
         signals.page_generator_finalized.send(self)
 
     def generate_output(self, writer):
         for page in chain(self.translations, self.pages,
-                            self.hidden_translations, self.hidden_pages):
-            writer.write_file(page.save_as, self.get_template(page.template),
-                    self.context, page=page,
-                    relative_urls=self.settings['RELATIVE_URLS'],
-                    override_output=hasattr(page, 'override_save_as'))
+                          self.hidden_translations, self.hidden_pages):
+            writer.write_file(
+                page.save_as, self.get_template(page.template),
+                self.context, page=page,
+                relative_urls=self.settings['RELATIVE_URLS'],
+                override_output=hasattr(page, 'override_save_as'))
 
 
 class StaticGenerator(Generator):
     """copy static paths (what you want to copy, like images, medias etc.
     to output"""
 
+    def __init__(self, *args, **kwargs):
+        super(StaticGenerator, self).__init__(*args, **kwargs)
+        signals.static_generator_init.send(self)
+
     def _copy_paths(self, paths, source, destination, output_path,
-            final_path=None):
+                    final_path=None):
         """Copy all the paths from source to destination"""
         for path in paths:
-            copy(path, source, os.path.join(output_path, destination),
-                 final_path)
+            if final_path:
+                copy(os.path.join(source, path),
+                     os.path.join(output_path, destination, final_path))
+            else:
+                copy(os.path.join(source, path),
+                     os.path.join(output_path, destination, path))
 
     def generate_context(self):
         self.staticfiles = []
-
-        # walk static paths
-        for static_path in self.settings['STATIC_PATHS']:
-            for f in self.get_files(
-                    static_path, extensions=False):
-                static = self.readers.read_file(
-                    base_path=self.path, path=f, content_class=Static,
-                    fmt='static', context=self.context,
-                    preread_signal=signals.static_generator_preread,
-                    preread_sender=self,
-                    context_signal=signals.static_generator_context,
-                    context_sender=self)
-                self.staticfiles.append(static)
-                self.add_source_path(static)
+        for f in self.get_files(self.settings['STATIC_PATHS'],
+                                extensions=False):
+            static = self.readers.read_file(
+                base_path=self.path, path=f, content_class=Static,
+                fmt='static', context=self.context,
+                preread_signal=signals.static_generator_preread,
+                preread_sender=self,
+                context_signal=signals.static_generator_context,
+                context_sender=self)
+            self.staticfiles.append(static)
+            self.add_source_path(static)
         self._update_context(('staticfiles',))
+        signals.static_generator_finalized.send(self)
 
     def generate_output(self, writer):
         self._copy_paths(self.settings['THEME_STATIC_PATHS'], self.theme,
@@ -573,11 +678,12 @@ class StaticGenerator(Generator):
             source_path = os.path.join(self.path, sc.source_path)
             save_as = os.path.join(self.output_path, sc.save_as)
             mkdir_p(os.path.dirname(save_as))
-            shutil.copy(source_path, save_as)
+            shutil.copy2(source_path, save_as)
             logger.info('copying {} to {}'.format(sc.source_path, sc.save_as))
 
 
 class SourceFileGenerator(Generator):
+
     def generate_context(self):
         self.output_extension = self.settings['OUTPUT_SOURCES_EXTENSION']
 
@@ -585,7 +691,7 @@ class SourceFileGenerator(Generator):
         output_path, _ = os.path.splitext(obj.save_as)
         dest = os.path.join(self.output_path,
                             output_path + self.output_extension)
-        copy('', obj.source_path, dest)
+        copy(obj.source_path, dest)
 
     def generate_output(self, writer=None):
         logger.info(' Generating source files...')
