@@ -1,55 +1,45 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals, print_function
 
-import datetime
 import logging
 import os
 import re
 
-try:
-    import docutils
-    import docutils.core
-    import docutils.io
-    from docutils.writers.html4css1 import HTMLTranslator
+import docutils
+import docutils.core
+import docutils.io
+from docutils.writers.html4css1 import HTMLTranslator
+import six
 
-    # import the directives to have pygments support
-    from pelican import rstdirectives  # NOQA
-except ImportError:
-    docutils = False
+# import the directives to have pygments support
+from pelican import rstdirectives  # NOQA
 try:
     from markdown import Markdown
 except ImportError:
     Markdown = False  # NOQA
-try:
-    from asciidocapi import AsciiDocAPI
-    asciidoc = True
-except ImportError:
-    asciidoc = False
 try:
     from html import escape
 except ImportError:
     from cgi import escape
-try:
-    from html.parser import HTMLParser
-except ImportError:
-    from HTMLParser import HTMLParser
+from six.moves.html_parser import HTMLParser
 
 from pelican import signals
 from pelican.contents import Page, Category, Tag, Author
-from pelican.utils import get_date, pelican_open
+from pelican.utils import get_date, pelican_open, FileStampDataCacher, SafeDatetime
 
 
 METADATA_PROCESSORS = {
     'tags': lambda x, y: [Tag(tag, y) for tag in x.split(',')],
     'date': lambda x, y: get_date(x),
+    'modified': lambda x, y: get_date(x),
     'status': lambda x, y: x.strip(),
     'category': Category,
     'author': Author,
+    'authors': lambda x, y: [Author(author.strip(), y) for author in x.split(',')],
 }
 
 logger = logging.getLogger(__name__)
 
-
 class BaseReader(object):
     """Base class to read files.
 
@@ -128,6 +118,19 @@ class RstReader(BaseReader):
     enabled = bool(docutils)
     file_extensions = ['rst']
 
+    class FileInput(docutils.io.FileInput):
+        """Patch docutils.io.FileInput to remove "U" mode in py3.
+
+        Universal newlines is enabled by default and "U" mode is deprecated
+        in py3.
+
+        """
+
+        def __init__(self, *args, **kwargs):
+            if six.PY3:
+                kwargs['mode'] = kwargs.get('mode', 'r').replace('U', '')
+            docutils.io.FileInput.__init__(self, *args, **kwargs)
+
     def __init__(self, *args, **kwargs):
         super(RstReader, self).__init__(*args, **kwargs)
 
@@ -143,6 +146,10 @@ class RstReader(BaseReader):
                         value = render_node_to_html(document, body_elem)
                     else:
                         value = body_elem.astext()
+                elif element.tagname == 'authors':  # author list
+                    name = element.tagname
+                    value = [element.astext() for element in element.children]
+                    value = ','.join(value) # METADATA_PROCESSORS expects a string
                 else:  # standard fields (e.g. address)
                     name = element.tagname
                     value = element.astext()
@@ -154,18 +161,21 @@ class RstReader(BaseReader):
     def _get_publisher(self, source_path):
         extra_params = {'initial_header_level': '2',
                         'syntax_highlight': 'short',
-                        'input_encoding': 'utf-8'}
+                        'input_encoding': 'utf-8',
+                        'exit_status_level': 2,
+                        'embed_stylesheet': False}
         user_params = self.settings.get('DOCUTILS_SETTINGS')
         if user_params:
             extra_params.update(user_params)
 
         pub = docutils.core.Publisher(
+            source_class=self.FileInput,
             destination_class=docutils.io.StringOutput)
         pub.set_components('standalone', 'restructuredtext', 'html')
         pub.writer.translator_class = PelicanHTMLTranslator
         pub.process_programmatic_settings(None, extra_params, None)
         pub.set_source(source_path=source_path)
-        pub.publish()
+        pub.publish(enable_exit_status=True)
         return pub
 
     def read(self, source_path):
@@ -198,12 +208,18 @@ class MarkdownReader(BaseReader):
         for name, value in meta.items():
             name = name.lower()
             if name == "summary":
+                # handle summary metadata as markdown
+                # summary metadata is special case and join all list values
                 summary_values = "\n".join(value)
                 # reset the markdown instance to clear any state
                 self._md.reset()
                 summary = self._md.convert(summary_values)
                 output[name] = self.process_metadata(name, summary)
+            elif len(value) > 1:
+                # handle list metadata as list of string
+                output[name] = self.process_metadata(name, value)
             else:
+                # otherwise, handle metadata as single string
                 output[name] = self.process_metadata(name, value[0])
         return output
 
@@ -226,7 +242,11 @@ class HTMLReader(BaseReader):
 
     class _HTMLParser(HTMLParser):
         def __init__(self, settings, filename):
-            HTMLParser.__init__(self)
+            try:
+                # Python 3.4+
+                HTMLParser.__init__(self, convert_charrefs=False)
+            except TypeError:
+                HTMLParser.__init__(self)
             self.body = ''
             self.metadata = {}
             self.settings = settings
@@ -302,12 +322,21 @@ class HTMLReader(BaseReader):
             return result + '>'
 
         def _handle_meta_tag(self, attrs):
-            name = self._attr_value(attrs, 'name').lower()
+            name = self._attr_value(attrs, 'name')
+            if name is None:
+                attr_serialized = ', '.join(['{}="{}"'.format(k, v) for k, v in attrs])
+                logger.warning("Meta tag in file %s does not have a 'name' attribute, skipping. Attributes: %s", self._filename, attr_serialized)
+                return
+            name = name.lower()
             contents = self._attr_value(attrs, 'content', '')
             if not contents:
                 contents = self._attr_value(attrs, 'contents', '')
                 if contents:
-                    logger.warning("Meta tag attribute 'contents' used in file %s, should be changed to 'content'", self._filename)
+                    logger.warning((
+                        "Meta tag attribute 'contents' used in file {}, should"
+                        " be changed to 'content'".format(self._filename),
+                        "Other files have meta tag attribute 'contents' that"
+                        " should be changed to 'content'"))
 
             if name == 'keywords':
                 name = 'tags'
@@ -330,41 +359,7 @@ class HTMLReader(BaseReader):
         return parser.body, metadata
 
 
-class AsciiDocReader(BaseReader):
-    """Reader for AsciiDoc files"""
-
-    enabled = bool(asciidoc)
-    file_extensions = ['asc']
-    default_options = ["--no-header-footer", "-a newline=\\n"]
-
-    def read(self, source_path):
-        """Parse content and metadata of asciidoc files"""
-        from cStringIO import StringIO
-        with pelican_open(source_path) as source:
-            text = StringIO(source)
-        content = StringIO()
-        ad = AsciiDocAPI()
-
-        options = self.settings['ASCIIDOC_OPTIONS']
-        if isinstance(options, (str, unicode)):
-            options = [m.strip() for m in options.split(',')]
-        options = self.default_options + options
-        for o in options:
-            ad.options(*o.split())
-
-        ad.execute(text, content, backend="html4")
-        content = content.getvalue()
-
-        metadata = {}
-        for name, value in ad.asciidoc.document.attributes.items():
-            name = name.lower()
-            metadata[name] = self.process_metadata(name, value)
-        if 'doctitle' in metadata:
-            metadata['title'] = metadata['doctitle']
-        return content, metadata
-
-
-class Readers(object):
+class Readers(FileStampDataCacher):
     """Interface for all readers.
 
     This class contains a mapping of file extensions / Reader classes, to know
@@ -374,27 +369,20 @@ class Readers(object):
 
     """
 
-    # used to warn about missing dependencies only once, at the first
-    # instanciation of a Readers object.
-    warn_missing_deps = True
-
-    def __init__(self, settings=None):
+    def __init__(self, settings=None, cache_name=''):
         self.settings = settings or {}
         self.readers = {}
         self.reader_classes = {}
 
         for cls in [BaseReader] + BaseReader.__subclasses__():
             if not cls.enabled:
-                if self.__class__.warn_missing_deps:
-                    logger.debug('Missing dependencies for {}'
-                                 .format(', '.join(cls.file_extensions)))
+                logger.debug('Missing dependencies for {}'
+                             .format(', '.join(cls.file_extensions)))
                 continue
 
             for ext in cls.file_extensions:
                 self.reader_classes[ext] = cls
 
-        self.__class__.warn_missing_deps = False
-
         if self.settings['READERS']:
             self.reader_classes.update(self.settings['READERS'])
 
@@ -406,6 +394,15 @@ class Readers(object):
 
             self.readers[fmt] = reader_class(self.settings)
 
+        # set up caching
+        cache_this_level = (cache_name != '' and
+                            self.settings['CONTENT_CACHING_LAYER'] == 'reader')
+        caching_policy = cache_this_level and self.settings['CACHE_CONTENT']
+        load_policy = cache_this_level and self.settings['LOAD_CONTENT_CACHE']
+        super(Readers, self).__init__(settings, cache_name,
+                                      caching_policy, load_policy,
+                                      )
+
     @property
     def extensions(self):
         return self.readers.keys()
@@ -444,7 +441,10 @@ class Readers(object):
             source_path=source_path, settings=self.settings,
             process=reader.process_metadata))
 
-        content, reader_metadata = reader.read(path)
+        content, reader_metadata = self.get_cached_data(path, (None, None))
+        if content is None:
+            content, reader_metadata = reader.read(path)
+            self.cache_data(path, (content, reader_metadata))
         metadata.update(reader_metadata)
 
         if content:
@@ -452,10 +452,13 @@ class Readers(object):
             find_empty_alt(content, path)
 
         # eventually filter the content with typogrify if asked so
-        if content and self.settings['TYPOGRIFY']:
+        if self.settings['TYPOGRIFY']:
             from typogrify.filters import typogrify
-            content = typogrify(content)
-            metadata['title'] = typogrify(metadata['title'])
+            if content:
+                content = typogrify(content)
+                metadata['title'] = typogrify(metadata['title'])
+            if 'summary' in metadata:
+                metadata['summary'] = typogrify(metadata['summary'])
 
         if context_signal:
             logger.debug('signal {}.send({}, <metadata>)'.format(
@@ -491,19 +494,10 @@ def find_empty_alt(content, path):
             src=(['"])(.*)\5
         )
         """, re.X)
-    matches = re.findall(imgs, content)
-    # find a correct threshold
-    nb_warnings = 10
-    if len(matches) == nb_warnings + 1:
-        nb_warnings += 1  # avoid bad looking case
-    # print one warning per image with empty alt until threshold
-    for match in matches[:nb_warnings]:
-        logger.warning('Empty alt attribute for image {} in {}'.format(
-            os.path.basename(match[1] + match[5]), path))
-    # print one warning for the other images with empty alt
-    if len(matches) > nb_warnings:
-        logger.warning('{} other images with empty alt attributes'
-                       .format(len(matches) - nb_warnings))
+    for match in re.findall(imgs, content):
+        logger.warning(('Empty alt attribute for image {} in {}'.format(
+            os.path.basename(match[1] + match[5]), path),
+            'Other images have empty alt attributes'))
 
 
 def default_metadata(settings=None, process=None):
@@ -514,8 +508,8 @@ def default_metadata(settings=None, process=None):
             if process:
                 value = process('category', value)
             metadata['category'] = value
-        if 'DEFAULT_DATE' in settings and settings['DEFAULT_DATE'] != 'fs':
-            metadata['date'] = datetime.datetime(*settings['DEFAULT_DATE'])
+        if settings.get('DEFAULT_DATE', None) and settings['DEFAULT_DATE'] != 'fs':
+            metadata['date'] = SafeDatetime(*settings['DEFAULT_DATE'])
     return metadata
 
 
@@ -523,7 +517,7 @@ def path_metadata(full_path, source_path, settings=None):
     metadata = {}
     if settings:
         if settings.get('DEFAULT_DATE', None) == 'fs':
-            metadata['date'] = datetime.datetime.fromtimestamp(
+            metadata['date'] = SafeDatetime.fromtimestamp(
                 os.stat(full_path).st_ctime)
         metadata.update(settings.get('EXTRA_PATH_METADATA', {}).get(
             source_path, {}))
@@ -546,7 +540,7 @@ def parse_path_metadata(source_path, settings=None, process=None):
     ...     process=reader.process_metadata)
     >>> pprint.pprint(metadata)  # doctest: +ELLIPSIS
     {'category': <pelican.urlwrappers.Category object at ...>,
-     'date': datetime.datetime(2013, 1, 1, 0, 0),
+     'date': SafeDatetime(2013, 1, 1, 0, 0),
      'slug': 'my-slug'}
     """
     metadata = {}
