@@ -6,6 +6,7 @@ from blinker import signal
 
 initialized = signal('pelican_initialized')
 get_generators = signal('get_generators')
+get_writer = signal('get_writer')
 finalized = signal('pelican_finalized')
 
 # Reader-level signals
@@ -17,7 +18,10 @@ readers_init = signal('readers_init')
 generator_init = signal('generator_init')
 
 article_generator_init = signal('article_generator_init')
+article_generator_pretaxonomy = signal('article_generator_pretaxonomy')
 article_generator_finalized = signal('article_generator_finalized')
+article_generator_write_article = signal('article_generator_write_article')
+article_writer_finalized = signal('article_writer_finalized')
 
 page_generator_init = signal('page_generator_init')
 page_generator_finalized = signal('page_generator_finalized')
