@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals, print_function
 
-import datetime
 import os
 
 from pelican import readers
+from pelican.utils import SafeDatetime
 from pelican.tests.support import unittest, get_settings
 
 CUR_DIR = os.path.dirname(__file__)
@@ -40,8 +40,10 @@ class RstReaderTest(ReaderTest):
             'title': 'This is a super article !',
             'summary': '<p class="first last">Multi-line metadata should be'
                        ' supported\nas well as <strong>inline'
-                       ' markup</strong>.</p>\n',
-            'date': datetime.datetime(2010, 12, 2, 10, 14),
+                       ' markup</strong> and stuff to &quot;typogrify'
+                       '&quot;...</p>\n',
+            'date': SafeDatetime(2010, 12, 2, 10, 14),
+            'modified': SafeDatetime(2010, 12, 2, 10, 20),
             'tags': ['foo', 'bar', 'foobar'],
             'custom_field': 'http://notmyidea.org',
         }
@@ -68,7 +70,7 @@ class RstReaderTest(ReaderTest):
             'category': 'yeah',
             'author': 'Alexis Métaireau',
             'title': 'Rst with filename metadata',
-            'date': datetime.datetime(2012, 11, 29),
+            'date': SafeDatetime(2012, 11, 29),
         }
         for key, value in page.metadata.items():
             self.assertEqual(value, expected[key], key)
@@ -83,7 +85,7 @@ class RstReaderTest(ReaderTest):
             'category': 'yeah',
             'author': 'Alexis Métaireau',
             'title': 'Rst with filename metadata',
-            'date': datetime.datetime(2012, 11, 29),
+            'date': SafeDatetime(2012, 11, 29),
             'slug': 'article_with_filename_metadata',
             'mymeta': 'foo',
         }
@@ -104,8 +106,8 @@ class RstReaderTest(ReaderTest):
         # if nothing is specified in the settings, the content should be
         # unmodified
         page = self.read_file(path='article.rst')
-        expected = ('<p>This is some content. With some stuff to '
-                    '&quot;typogrify&quot;.</p>\n<p>Now with added '
+        expected = ('<p>THIS is some content. With some stuff to '
+                    '&quot;typogrify&quot;...</p>\n<p>Now with added '
                     'support for <abbr title="three letter acronym">'
                     'TLA</abbr>.</p>\n')
 
@@ -114,15 +116,49 @@ class RstReaderTest(ReaderTest):
         try:
             # otherwise, typogrify should be applied
             page = self.read_file(path='article.rst', TYPOGRIFY=True)
-            expected = ('<p>This is some content. With some stuff to&nbsp;'
-                        '&#8220;typogrify&#8221;.</p>\n<p>Now with added '
-                        'support for <abbr title="three letter acronym">'
-                        '<span class="caps">TLA</span></abbr>.</p>\n')
+            expected = (
+                '<p><span class="caps">THIS</span> is some content. '
+                'With some stuff to&nbsp;&quot;typogrify&quot;&#8230;</p>\n'
+                '<p>Now with added support for <abbr title="three letter '
+                'acronym"><span class="caps">TLA</span></abbr>.</p>\n')
 
             self.assertEqual(page.content, expected)
         except ImportError:
             return unittest.skip('need the typogrify distribution')
 
+    def test_typogrify_summary(self):
+        # if nothing is specified in the settings, the summary should be
+        # unmodified
+        page = self.read_file(path='article_with_metadata.rst')
+        expected = ('<p class="first last">Multi-line metadata should be'
+                    ' supported\nas well as <strong>inline'
+                    ' markup</strong> and stuff to &quot;typogrify'
+                    '&quot;...</p>\n')
+
+        self.assertEqual(page.metadata['summary'], expected)
+
+        try:
+            # otherwise, typogrify should be applied
+            page = self.read_file(path='article_with_metadata.rst',
+                                  TYPOGRIFY=True)
+            expected = ('<p class="first last">Multi-line metadata should be'
+                        ' supported\nas well as <strong>inline'
+                        ' markup</strong> and stuff to&nbsp;&quot;typogrify'
+                        '&quot;&#8230;</p>\n')
+
+            self.assertEqual(page.metadata['summary'], expected)
+        except ImportError:
+            return unittest.skip('need the typogrify distribution')
+
+    def test_article_with_multiple_authors(self):
+        page = self.read_file(path='article_with_multiple_authors.rst')
+        expected = {
+            'authors': ['First Author', 'Second Author']
+        }
+
+        for key, value in expected.items():
+            self.assertEqual(value, page.metadata[key], key)
+
 
 class MdReaderTest(ReaderTest):
 
@@ -135,7 +171,8 @@ class MdReaderTest(ReaderTest):
             'category': 'test',
             'title': 'Test md File',
             'summary': '<p>I have a lot to test</p>',
-            'date': datetime.datetime(2010, 12, 2, 10, 14),
+            'date': SafeDatetime(2010, 12, 2, 10, 14),
+            'modified': SafeDatetime(2010, 12, 2, 10, 20),
             'tags': ['foo', 'bar', 'foobar'],
         }
         for key, value in metadata.items():
@@ -147,7 +184,8 @@ class MdReaderTest(ReaderTest):
             'title': 'マックOS X 10.8でパイソンとVirtualenvをインストールと設定',
             'summary': '<p>パイソンとVirtualenvをまっくでインストールする方法について明確に説明します。</p>',
             'category': '指導書',
-            'date': datetime.datetime(2012, 12, 20),
+            'date': SafeDatetime(2012, 12, 20),
+            'modified': SafeDatetime(2012, 12, 22),
             'tags': ['パイソン', 'マック'],
             'slug': 'python-virtualenv-on-mac-osx-mountain-lion-10.8',
         }
@@ -182,8 +220,17 @@ class MdReaderTest(ReaderTest):
             'summary': (
                 '<p>Summary with <strong>inline</strong> markup '
                 '<em>should</em> be supported.</p>'),
-            'date': datetime.datetime(2012, 10, 31),
+            'date': SafeDatetime(2012, 10, 31),
+            'modified': SafeDatetime(2012, 11, 1),
             'slug': 'article-with-markdown-containing-footnotes',
+            'multiline': [
+                'Line Metadata should be handle properly.',
+                'See syntax of Meta-Data extension of Python Markdown package:',
+                'If a line is indented by 4 or more spaces,',
+                'that line is assumed to be an additional line of the value',
+                'for the previous keyword.',
+                'A keyword may have as many lines as desired.',
+            ]
         }
         self.assertEqual(content, expected_content)
         for key, value in metadata.items():
@@ -264,7 +311,7 @@ class MdReaderTest(ReaderTest):
         expected = {
             'category': 'yeah',
             'author': 'Alexis Métaireau',
-            'date': datetime.datetime(2012, 11, 30),
+            'date': SafeDatetime(2012, 11, 30),
         }
         for key, value in expected.items():
             self.assertEqual(value, page.metadata[key], key)
@@ -278,7 +325,7 @@ class MdReaderTest(ReaderTest):
         expected = {
             'category': 'yeah',
             'author': 'Alexis Métaireau',
-            'date': datetime.datetime(2012, 11, 30),
+            'date': SafeDatetime(2012, 11, 30),
             'slug': 'md_w_filename_meta',
             'mymeta': 'foo',
         }
@@ -286,42 +333,6 @@ class MdReaderTest(ReaderTest):
             self.assertEqual(value, page.metadata[key], key)
 
 
-class AdReaderTest(ReaderTest):
-
-    @unittest.skipUnless(readers.asciidoc, "asciidoc isn't installed")
-    def test_article_with_asc_extension(self):
-        # Ensure the asc extension is being processed by the correct reader
-        page = self.read_file(
-            path='article_with_asc_extension.asc')
-        expected = ('<hr>\n<h2><a name="_used_for_pelican_test">'
-                    '</a>Used for pelican test</h2>\n'
-                    '<p>The quick brown fox jumped over'
-                    ' the lazy dog&#8217;s back.</p>\n')
-        self.assertEqual(page.content, expected)
-        expected = {
-            'category': 'Blog',
-            'author': 'Author O. Article',
-            'title': 'Test AsciiDoc File Header',
-            'date': datetime.datetime(2011, 9, 15, 9, 5),
-            'tags': ['Linux', 'Python', 'Pelican'],
-        }
-
-        for key, value in expected.items():
-            self.assertEqual(value, page.metadata[key], key)
-
-    @unittest.skipUnless(readers.asciidoc, "asciidoc isn't installed")
-    def test_article_with_asc_options(self):
-        # test to ensure the ASCIIDOC_OPTIONS is being used
-        reader = readers.AsciiDocReader(
-            dict(ASCIIDOC_OPTIONS=["-a revision=1.0.42"]))
-        content, metadata = reader.read(_path('article_with_asc_options.asc'))
-        expected = ('<hr>\n<h2><a name="_used_for_pelican_test"></a>Used for'
-                    ' pelican test</h2>\n<p>version 1.0.42</p>\n'
-                    '<p>The quick brown fox jumped over the lazy'
-                    ' dog&#8217;s back.</p>\n')
-        self.assertEqual(content, expected)
-
-
 class HTMLReaderTest(ReaderTest):
     def test_article_with_comments(self):
         page = self.read_file(path='article_with_comments.html')
@@ -347,7 +358,7 @@ class HTMLReaderTest(ReaderTest):
             'author': 'Alexis Métaireau',
             'title': 'This is a super article !',
             'summary': 'Summary and stuff',
-            'date': datetime.datetime(2010, 12, 2, 10, 14),
+            'date': SafeDatetime(2010, 12, 2, 10, 14),
             'tags': ['foo', 'bar', 'foobar'],
             'custom_field': 'http://notmyidea.org',
         }
@@ -355,6 +366,15 @@ class HTMLReaderTest(ReaderTest):
         for key, value in expected.items():
             self.assertEqual(value, page.metadata[key], key)
 
+    def test_article_with_multiple_authors(self):
+        page = self.read_file(path='article_with_multiple_authors.html')
+        expected = {
+            'authors': ['First Author', 'Second Author']
+        }
+
+        for key, value in expected.items():
+            self.assertEqual(value, page.metadata[key], key)
+
     def test_article_with_metadata_and_contents_attrib(self):
         page = self.read_file(path='article_with_metadata_and_contents.html')
         expected = {
@@ -362,7 +382,7 @@ class HTMLReaderTest(ReaderTest):
             'author': 'Alexis Métaireau',
             'title': 'This is a super article !',
             'summary': 'Summary and stuff',
-            'date': datetime.datetime(2010, 12, 2, 10, 14),
+            'date': SafeDatetime(2010, 12, 2, 10, 14),
             'tags': ['foo', 'bar', 'foobar'],
             'custom_field': 'http://notmyidea.org',
         }
@@ -383,3 +403,13 @@ class HTMLReaderTest(ReaderTest):
         self.assertIn('category', page.metadata, 'Key should be lowercase.')
         self.assertEqual('Yeah', page.metadata.get('category'),
                          'Value keeps cases.')
+
+    def test_article_with_nonconformant_meta_tags(self):
+        page = self.read_file(path='article_with_nonconformant_meta_tags.html')
+        expected = {
+            'summary': 'Summary and stuff',
+            'title': 'Article with Nonconformant HTML meta tags',
+        }
+
+        for key, value in expected.items():
+            self.assertEqual(value, page.metadata[key], key)
