@@ -1,9 +1,8 @@
 # -*- coding: utf-8 -*-
-from __future__ import unicode_literals, print_function
+from __future__ import unicode_literals, print_function, absolute_import
 import logging
 import shutil
 import os
-import datetime
 import time
 import locale
 from sys import platform, version_info
@@ -15,7 +14,7 @@ from pelican.generators import TemplatePagesGenerator
 from pelican.writers import Writer
 from pelican.settings import read_settings
 from pelican import utils
-from .support import get_article, LoggedTestCase, locale_available, unittest
+from pelican.tests.support import get_article, LoggedTestCase, locale_available, unittest
 
 
 class TestUtils(LoggedTestCase):
@@ -38,18 +37,24 @@ class TestUtils(LoggedTestCase):
 
     def test_get_date(self):
         # valid ones
-        date = datetime.datetime(year=2012, month=11, day=22)
-        date_hour = datetime.datetime(
+        date = utils.SafeDatetime(year=2012, month=11, day=22)
+        date_hour = utils.SafeDatetime(
             year=2012, month=11, day=22, hour=22, minute=11)
-        date_hour_sec = datetime.datetime(
+        date_hour_z = utils.SafeDatetime(
+            year=2012, month=11, day=22, hour=22, minute=11,
+            tzinfo=pytz.timezone('UTC'))
+        date_hour_est = utils.SafeDatetime(
+            year=2012, month=11, day=22, hour=22, minute=11,
+            tzinfo=pytz.timezone('EST'))
+        date_hour_sec = utils.SafeDatetime(
             year=2012, month=11, day=22, hour=22, minute=11, second=10)
-        date_hour_sec_z = datetime.datetime(
+        date_hour_sec_z = utils.SafeDatetime(
             year=2012, month=11, day=22, hour=22, minute=11, second=10,
             tzinfo=pytz.timezone('UTC'))
-        date_hour_sec_est = datetime.datetime(
+        date_hour_sec_est = utils.SafeDatetime(
             year=2012, month=11, day=22, hour=22, minute=11, second=10,
             tzinfo=pytz.timezone('EST'))
-        date_hour_sec_frac_z = datetime.datetime(
+        date_hour_sec_frac_z = utils.SafeDatetime(
             year=2012, month=11, day=22, hour=22, minute=11, second=10,
             microsecond=123000, tzinfo=pytz.timezone('UTC'))
         dates = {
@@ -61,22 +66,42 @@ class TestUtils(LoggedTestCase):
             '22/11/2012': date,
             '22.11.2012': date,
             '22.11.2012 22:11': date_hour,
+            '2012-11-22T22:11Z': date_hour_z,
+            '2012-11-22T22:11-0500': date_hour_est,
             '2012-11-22 22:11:10': date_hour_sec,
             '2012-11-22T22:11:10Z': date_hour_sec_z,
             '2012-11-22T22:11:10-0500': date_hour_sec_est,
             '2012-11-22T22:11:10.123Z': date_hour_sec_frac_z,
             }
 
+        # examples from http://www.w3.org/TR/NOTE-datetime
+        iso_8601_date = utils.SafeDatetime(year=1997, month=7, day=16)
+        iso_8601_date_hour_tz = utils.SafeDatetime(
+            year=1997, month=7, day=16, hour=19, minute=20,
+            tzinfo=pytz.timezone('CET'))
+        iso_8601_date_hour_sec_tz = utils.SafeDatetime(
+            year=1997, month=7, day=16, hour=19, minute=20, second=30,
+            tzinfo=pytz.timezone('CET'))
+        iso_8601_date_hour_sec_ms_tz = utils.SafeDatetime(
+            year=1997, month=7, day=16, hour=19, minute=20, second=30,
+            microsecond=450000, tzinfo=pytz.timezone('CET'))
+        iso_8601 = {
+            '1997-07-16': iso_8601_date,
+            '1997-07-16T19:20+01:00': iso_8601_date_hour_tz,
+            '1997-07-16T19:20:30+01:00': iso_8601_date_hour_sec_tz,
+            '1997-07-16T19:20:30.45+01:00': iso_8601_date_hour_sec_ms_tz,
+        }
+
         # invalid ones
         invalid_dates = ['2010-110-12', 'yay']
 
-        if version_info < (3, 2):
-            dates.pop('2012-11-22T22:11:10-0500')
-            invalid_dates.append('2012-11-22T22:11:10-0500')
 
         for value, expected in dates.items():
             self.assertEqual(utils.get_date(value), expected, value)
 
+        for value, expected in iso_8601.items():
+            self.assertEqual(utils.get_date(value), expected, value)
+
         for item in invalid_dates:
             self.assertRaises(ValueError, utils.get_date, item)
 
@@ -232,7 +257,7 @@ class TestUtils(LoggedTestCase):
         self.assertFalse(os.path.exists(test_directory))
 
     def test_strftime(self):
-        d = datetime.date(2012, 8, 29)
+        d = utils.SafeDatetime(2012, 8, 29)
 
         # simple formatting
         self.assertEqual(utils.strftime(d, '%d/%m/%y'), '29/08/12')
@@ -270,7 +295,7 @@ class TestUtils(LoggedTestCase):
         else:
             locale.setlocale(locale.LC_TIME, str('tr_TR.UTF-8'))
 
-        d = datetime.date(2012, 8, 29)
+        d = utils.SafeDatetime(2012, 8, 29)
 
         # simple
         self.assertEqual(utils.strftime(d, '%d %B %Y'), '29 Ağustos 2012')
@@ -303,7 +328,7 @@ class TestUtils(LoggedTestCase):
         else:
             locale.setlocale(locale.LC_TIME, str('fr_FR.UTF-8'))
 
-        d = datetime.date(2012, 8, 29)
+        d = utils.SafeDatetime(2012, 8, 29)
 
         # simple
         self.assertEqual(utils.strftime(d, '%d %B %Y'), '29 août 2012')
@@ -323,6 +348,90 @@ class TestUtils(LoggedTestCase):
         locale.setlocale(locale.LC_TIME, old_locale)
 
 
+class TestCopy(unittest.TestCase):
+    '''Tests the copy utility'''
+
+    def setUp(self):
+        self.root_dir = mkdtemp(prefix='pelicantests.')
+        self.old_locale = locale.setlocale(locale.LC_ALL)
+        locale.setlocale(locale.LC_ALL, str('C'))
+
+    def tearDown(self):
+        shutil.rmtree(self.root_dir)
+        locale.setlocale(locale.LC_ALL, self.old_locale)
+
+    def _create_file(self, *path):
+        with open(os.path.join(self.root_dir, *path), 'w') as f:
+            f.write('42\n')
+
+    def _create_dir(self, *path):
+        os.makedirs(os.path.join(self.root_dir, *path))
+
+    def _exist_file(self, *path):
+        path = os.path.join(self.root_dir, *path)
+        self.assertTrue(os.path.isfile(path), 'File does not exist: %s' % path)
+
+    def _exist_dir(self, *path):
+        path = os.path.join(self.root_dir, *path)
+        self.assertTrue(os.path.exists(path),
+                        'Directory does not exist: %s' % path)
+
+    def test_copy_file_same_path(self):
+        self._create_file('a.txt')
+        utils.copy(os.path.join(self.root_dir, 'a.txt'),
+                   os.path.join(self.root_dir, 'b.txt'))
+        self._exist_file('b.txt')
+
+    def test_copy_file_different_path(self):
+        self._create_dir('a')
+        self._create_dir('b')
+        self._create_file('a', 'a.txt')
+        utils.copy(os.path.join(self.root_dir, 'a', 'a.txt'),
+                   os.path.join(self.root_dir, 'b', 'b.txt'))
+        self._exist_dir('b')
+        self._exist_file('b', 'b.txt')
+
+    def test_copy_file_create_dirs(self):
+        self._create_file('a.txt')
+        utils.copy(os.path.join(self.root_dir, 'a.txt'),
+                   os.path.join(self.root_dir, 'b0', 'b1', 'b2', 'b3', 'b.txt'))
+        self._exist_dir('b0')
+        self._exist_dir('b0', 'b1')
+        self._exist_dir('b0', 'b1', 'b2')
+        self._exist_dir('b0', 'b1', 'b2', 'b3')
+        self._exist_file('b0', 'b1', 'b2', 'b3', 'b.txt')
+
+    def test_copy_dir_same_path(self):
+        self._create_dir('a')
+        self._create_file('a', 'a.txt')
+        utils.copy(os.path.join(self.root_dir, 'a'),
+                   os.path.join(self.root_dir, 'b'))
+        self._exist_dir('b')
+        self._exist_file('b', 'a.txt')
+
+    def test_copy_dir_different_path(self):
+        self._create_dir('a0')
+        self._create_dir('a0', 'a1')
+        self._create_file('a0', 'a1', 'a.txt')
+        self._create_dir('b0')
+        utils.copy(os.path.join(self.root_dir, 'a0', 'a1'),
+                   os.path.join(self.root_dir, 'b0', 'b1'))
+        self._exist_dir('b0', 'b1')
+        self._exist_file('b0', 'b1', 'a.txt')
+
+    def test_copy_dir_create_dirs(self):
+        self._create_dir('a')
+        self._create_file('a', 'a.txt')
+        utils.copy(os.path.join(self.root_dir, 'a'),
+                   os.path.join(self.root_dir, 'b0', 'b1', 'b2', 'b3', 'b'))
+        self._exist_dir('b0')
+        self._exist_dir('b0', 'b1')
+        self._exist_dir('b0', 'b1', 'b2')
+        self._exist_dir('b0', 'b1', 'b2', 'b3')
+        self._exist_dir('b0', 'b1', 'b2', 'b3', 'b')
+        self._exist_file('b0', 'b1', 'b2', 'b3', 'b', 'a.txt')
+
+
 class TestDateFormatter(unittest.TestCase):
     '''Tests that the output of DateFormatter jinja filter is same as
     utils.strftime'''
@@ -338,7 +447,7 @@ class TestDateFormatter(unittest.TestCase):
         os.makedirs(template_dir)
         with open(template_path, 'w') as template_file:
             template_file.write('date = {{ date|strftime("%A, %d %B %Y") }}')
-        self.date = datetime.date(2012, 8, 29)
+        self.date = utils.SafeDatetime(2012, 8, 29)
 
 
     def tearDown(self):
@@ -348,6 +457,25 @@ class TestDateFormatter(unittest.TestCase):
         locale.setlocale(locale.LC_ALL, '')
 
 
+    @unittest.skipUnless(locale_available('fr_FR.UTF-8') or
+                         locale_available('French'),
+                         'French locale needed')
+    def test_french_strftime(self):
+        # This test tries to reproduce an issue that occured with python3.3 under macos10 only
+        locale.setlocale(locale.LC_ALL, str('fr_FR.UTF-8'))
+        date = utils.SafeDatetime(2014,8,14)
+        # we compare the lower() dates since macos10 returns "Jeudi" for %A whereas linux reports "jeudi"
+        self.assertEqual( u'jeudi, 14 août 2014', utils.strftime(date, date_format="%A, %d %B %Y").lower() )
+        df = utils.DateFormatter()
+        self.assertEqual( u'jeudi, 14 août 2014', df(date, date_format="%A, %d %B %Y").lower() )
+        # Let us now set the global locale to C:
+        locale.setlocale(locale.LC_ALL, str('C'))
+        # DateFormatter should still work as expected since it is the whole point of DateFormatter
+        # (This is where pre-2014/4/15 code fails on macos10)
+        df_date = df(date, date_format="%A, %d %B %Y").lower()
+        self.assertEqual( u'jeudi, 14 août 2014', df_date )
+
+
     @unittest.skipUnless(locale_available('fr_FR.UTF-8') or
                          locale_available('French'),
                          'French locale needed')
