@@ -4,11 +4,10 @@
 from __future__ import unicode_literals, print_function
 import argparse
 try:
-    # py3k import
-    from html.parser import HTMLParser
+    from html import unescape  # py3.4+
 except ImportError:
-    # py2 import
-    from HTMLParser import HTMLParser  # NOQA
+    from six.moves.html_parser import HTMLParser
+    unescape = HTMLParser().unescape
 import os
 import re
 import subprocess
@@ -17,8 +16,11 @@ import time
 import logging
 
 from codecs import open
+from six.moves.urllib.error import URLError
+from six.moves.urllib.parse import urlparse
+from six.moves.urllib.request import urlretrieve
 
-from pelican.utils import slugify
+from pelican.utils import slugify, SafeDatetime
 from pelican.log import init
 
 logger = logging.getLogger(__name__)
@@ -96,50 +98,71 @@ def decode_wp_content(content, br=True):
 
     return content
 
-
-def wp2fields(xml):
-    """Opens a wordpress XML file, and yield pelican fields"""
+def get_items(xml):
+    """Opens a wordpress xml file and returns a list of items"""
     try:
         from bs4 import BeautifulSoup
     except ImportError:
         error = ('Missing dependency '
                  '"BeautifulSoup4" and "lxml" required to import Wordpress XML files.')
         sys.exit(error)
-
-
     with open(xml, encoding='utf-8') as infile:
         xmlfile = infile.read()
     soup = BeautifulSoup(xmlfile, "xml")
     items = soup.rss.channel.findAll('item')
+    return items
 
+def get_filename(filename, post_id):
+    if filename is not None:
+        return filename
+    else:
+        return post_id
+
+def wp2fields(xml, wp_custpost=False):
+    """Opens a wordpress XML file, and yield Pelican fields"""
+
+    items = get_items(xml)
     for item in items:
 
         if item.find('status').string == "publish":
 
             try:
                 # Use HTMLParser due to issues with BeautifulSoup 3
-                title = HTMLParser().unescape(item.title.contents[0])
+                title = unescape(item.title.contents[0])
             except IndexError:
                 title = 'No title [%s]' % item.find('post_name').string
-                logger.warn('Post "%s" is lacking a proper title' % title)
+                logger.warning('Post "%s" is lacking a proper title' % title)
+
+            filename = item.find('post_name').string
+            post_id = item.find('post_id').string
+            filename = get_filename(filename, post_id)
 
             content = item.find('encoded').string
-            filename = item.find('post_name').string
-
             raw_date = item.find('post_date').string
             date_object = time.strptime(raw_date, "%Y-%m-%d %H:%M:%S")
             date = time.strftime("%Y-%m-%d %H:%M", date_object)
             author = item.find('creator').string
- 
+
             categories = [cat.string for cat in item.findAll('category', {'domain' : 'category'})]
             # caturl = [cat['nicename'] for cat in item.find(domain='category')]
 
             tags = [tag.string for tag in item.findAll('category', {'domain' : 'post_tag'})]
 
             kind = 'article'
-            if item.find('post_type').string == 'page':
+            post_type = item.find('post_type').string
+            if post_type == 'page':
                 kind = 'page'
-
+            elif wp_custpost:
+                if post_type == 'post':
+                    pass
+                # Old behaviour was to name everything not a page as an article.
+                # Theoretically all attachments have status == inherit so
+                # no attachments should be here. But this statement is to
+                # maintain existing behaviour in case that doesn't hold true.
+                elif post_type == 'attachment':
+                    pass
+                else:
+                    kind = post_type
             yield (title, content, filename, date, author, categories, tags,
                    kind, "wp-html")
 
@@ -279,7 +302,7 @@ def dc2fields(file):
 def posterous2fields(api_token, email, password):
     """Imports posterous posts"""
     import base64
-    from datetime import datetime, timedelta
+    from datetime import timedelta
     try:
         # py3k import
         import json
@@ -316,7 +339,7 @@ def posterous2fields(api_token, email, password):
                 slug = slugify(post.get('title'))
             tags = [tag.get('name') for tag in post.get('tags')]
             raw_date = post.get('display_date')
-            date_object = datetime.strptime(raw_date[:-6], "%Y/%m/%d %H:%M:%S")
+            date_object = SafeDatetime.strptime(raw_date[:-6], "%Y/%m/%d %H:%M:%S")
             offset = int(raw_date[-5:])
             delta = timedelta(hours = offset / 100)
             date_object -= delta
@@ -407,7 +430,6 @@ def tumblr2fields(api_key, blogname):
         offset += len(posts)
         posts = get_tumblr_posts(api_key, blogname, offset)
 
-
 def feed2fields(file):
     """Read a feed and yield pelican fields"""
     import feedparser
@@ -423,10 +445,11 @@ def feed2fields(file):
         yield (entry.title, entry.description, slug, date, author, [], tags,
                kind, "html")
 
+def build_header(title, date, author, categories, tags, slug, attachments=None):
+    from docutils.utils import column_width
 
-def build_header(title, date, author, categories, tags, slug):
     """Build a header from a list of fields"""
-    header = '%s\n%s\n' % (title, '#' * len(title))
+    header = '%s\n%s\n' % (title, '#' * column_width(title))
     if date:
         header += ':date: %s\n' % date
     if author:
@@ -437,10 +460,13 @@ def build_header(title, date, author, categories, tags, slug):
         header += ':tags: %s\n' % ', '.join(tags)
     if slug:
         header += ':slug: %s\n' % slug
+    if attachments:
+        header += ':attachments: %s\n' % ', '.join(attachments)
     header += '\n'
     return header
 
-def build_markdown_header(title, date, author, categories, tags, slug):
+def build_markdown_header(title, date, author, categories, tags, slug,
+        attachments=None):
     """Build a header from a list of fields"""
     header = 'Title: %s\n' % title
     if date:
@@ -453,51 +479,172 @@ def build_markdown_header(title, date, author, categories, tags, slug):
         header += 'Tags: %s\n' % ', '.join(tags)
     if slug:
         header += 'Slug: %s\n' % slug
+    if attachments:
+        header += 'Attachments: %s\n' % ', '.join(attachments)
     header += '\n'
     return header
 
+def get_ext(out_markup, in_markup='html'):
+    if in_markup == 'markdown' or out_markup == 'markdown':
+        ext = '.md'
+    else:
+        ext = '.rst'
+    return ext
+
+def get_out_filename(output_path, filename, ext, kind,
+        dirpage, dircat, categories, wp_custpost):
+    filename = os.path.basename(filename)
+
+    # Enforce filename restrictions for various filesystems at once; see
+    # http://en.wikipedia.org/wiki/Filename#Reserved_characters_and_words
+    # we do not need to filter words because an extension will be appended
+    filename = re.sub(r'[<>:"/\\|?*^% ]', '-', filename) # invalid chars
+    filename = filename.lstrip('.') # should not start with a dot
+    if not filename:
+        filename = '_'
+    filename = filename[:249] # allow for 5 extra characters
+
+    out_filename = os.path.join(output_path, filename+ext)
+    # option to put page posts in pages/ subdirectory
+    if dirpage and kind == 'page':
+        pages_dir = os.path.join(output_path, 'pages')
+        if not os.path.isdir(pages_dir):
+            os.mkdir(pages_dir)
+        out_filename = os.path.join(pages_dir, filename+ext)
+    elif not dirpage and kind == 'page':
+        pass
+    # option to put wp custom post types in directories with post type
+    # names. Custom post types can also have categories so option to
+    # create subdirectories with category names
+    elif kind != 'article':
+        if wp_custpost:
+            typename = slugify(kind)
+        else:
+            typename = ''
+            kind = 'article'
+        if dircat and (len(categories) > 0):
+            catname = slugify(categories[0])
+        else:
+            catname = ''
+        out_filename = os.path.join(output_path, typename,
+            catname, filename+ext)
+        if not os.path.isdir(os.path.join(output_path, typename, catname)):
+            os.makedirs(os.path.join(output_path, typename, catname))
+    # option to put files in directories with categories names
+    elif dircat and (len(categories) > 0):
+        catname = slugify(categories[0])
+        out_filename = os.path.join(output_path, catname, filename+ext)
+        if not os.path.isdir(os.path.join(output_path, catname)):
+            os.mkdir(os.path.join(output_path, catname))
+
+    return out_filename
+
+def get_attachments(xml):
+    """returns a dictionary of posts that have attachments with a list
+    of the attachment_urls
+    """
+    items = get_items(xml)
+    names = {}
+    attachments = []
+
+    for item in items:
+        kind = item.find('post_type').string
+        filename = item.find('post_name').string
+        post_id = item.find('post_id').string
+
+        if kind == 'attachment':
+            attachments.append((item.find('post_parent').string,
+                item.find('attachment_url').string))
+        else:
+            filename = get_filename(filename, post_id)
+            names[post_id] = filename
+    attachedposts = {}
+    for parent, url in attachments:
+        try:
+            parent_name = names[parent]
+        except KeyError:
+            #attachment's parent is not a valid post
+            parent_name = None
+
+        try:
+            attachedposts[parent_name].append(url)
+        except KeyError:
+            attachedposts[parent_name] = []
+            attachedposts[parent_name].append(url)
+    return attachedposts
+
+def download_attachments(output_path, urls):
+    """Downloads wordpress attachments and returns a list of paths to
+    attachments that can be associated with a post (relative path to output
+    directory). Files that fail to download, will not be added to posts"""
+    locations = []
+    for url in urls:
+        path = urlparse(url).path
+        #teardown path and rebuild to negate any errors with
+        #os.path.join and leading /'s
+        path = path.split('/')
+        filename = path.pop(-1)
+        localpath = ''
+        for item in path:
+            localpath = os.path.join(localpath, item)
+        full_path = os.path.join(output_path, localpath)
+        if not os.path.exists(full_path):
+            os.makedirs(full_path)
+        print('downloading {}'.format(filename))
+        try:
+            urlretrieve(url, os.path.join(full_path, filename))
+            locations.append(os.path.join(localpath, filename))
+        except URLError as e:
+            error = ("No file could be downloaded from {}; Error {}"
+                    .format(url, e))
+            logger.warning(error)
+        except IOError as e: #Python 2.7 throws an IOError rather Than URLError
+            # For japanese, the error might look kind of like this:
+            # e = IOError( 'socket error', socket.error(111, u'\u63a5\u7d9a\u3092\u62d2\u5426\u3055\u308c\u307e\u3057\u305f') )
+            # and not be suitable to use in "{}".format(e) , raising UnicodeDecodeError
+            # (This is at least the case on my Fedora running Python 2.7.5 
+            # (default, Feb 19 2014, 13:47:28) [GCC 4.8.2 20131212 (Red Hat 4.8.2-7)] on linux2
+            try:
+                error = ("No file could be downloaded from {}; Error {}"
+                        .format(url, e))
+            except UnicodeDecodeError:
+                # For lack of a better log message because we could not decode e, let's use repr(e)
+                error = ("No file could be downloaded from {}; Error {}"
+                        .format(url, repr(e)))
+            logger.warning(error)
+    return locations
+
+
 def fields2pelican(fields, out_markup, output_path,
         dircat=False, strip_raw=False, disable_slugs=False,
-        dirpage=False, filename_template=None, filter_author=None):
+        dirpage=False, filename_template=None, filter_author=None,
+        wp_custpost=False, wp_attach=False, attachments=None):
     for (title, content, filename, date, author, categories, tags,
             kind, in_markup) in fields:
         if filter_author and filter_author != author:
             continue
         slug = not disable_slugs and filename or None
-        if (in_markup == "markdown") or (out_markup == "markdown") :
-            ext = '.md'
-            header = build_markdown_header(title, date, author, categories, tags, slug)
+
+        if wp_attach and attachments:
+            try:
+                urls = attachments[filename]
+                attached_files = download_attachments(output_path, urls)
+            except KeyError:
+                attached_files = None
+        else:
+            attached_files = None
+
+        ext = get_ext(out_markup, in_markup)
+        if ext == '.md':
+            header = build_markdown_header(title, date, author, categories,
+                    tags, slug, attached_files)
         else:
             out_markup = "rst"
-            ext = '.rst'
-            header = build_header(title, date, author, categories, tags, slug)
-
-        filename = os.path.basename(filename)
-
-        # Enforce filename restrictions for various filesystems at once; see
-        # http://en.wikipedia.org/wiki/Filename#Reserved_characters_and_words
-        # we do not need to filter words because an extension will be appended
-        filename = re.sub(r'[<>:"/\\|?*^% ]', '-', filename) # invalid chars
-        filename = filename.lstrip('.') # should not start with a dot
-        if not filename:
-            filename = '_'
-        filename = filename[:249] # allow for 5 extra characters
-
-        # option to put page posts in pages/ subdirectory
-        if dirpage and kind == 'page':
-            pages_dir = os.path.join(output_path, 'pages')
-            if not os.path.isdir(pages_dir):
-                os.mkdir(pages_dir)
-            out_filename = os.path.join(pages_dir, filename+ext)
-        # option to put files in directories with categories names
-        elif dircat and (len(categories) > 0):
-            catname = slugify(categories[0])
-            out_filename = os.path.join(output_path, catname, filename+ext)
-            if not os.path.isdir(os.path.join(output_path, catname)):
-                os.mkdir(os.path.join(output_path, catname))
-        else:
-            out_filename = os.path.join(output_path, filename+ext)
+            header = build_header(title, date, author, categories,
+                    tags, slug, attached_files)
 
+        out_filename = get_out_filename(output_path, filename, ext,
+                kind, dirpage, dircat, categories, wp_custpost)
         print(out_filename)
 
         if in_markup in ("html", "wp-html"):
@@ -517,7 +664,7 @@ def fields2pelican(fields, out_markup, output_path,
 
 
             parse_raw = '--parse-raw' if not strip_raw else ''
-            cmd = ('pandoc --normalize --reference-links {0} --from=html'
+            cmd = ('pandoc --normalize {0} --from=html'
                    ' --to={1} -o "{2}" "{3}"').format(
                     parse_raw, out_markup, out_filename, html_filename)
 
@@ -545,12 +692,16 @@ def fields2pelican(fields, out_markup, output_path,
 
         with open(out_filename, 'w', encoding='utf-8') as fs:
             fs.write(header + content)
-
+    if wp_attach and attachments and None in attachments:
+        print("downloading attachments that don't have a parent post")
+        urls = attachments[None]
+        orphan_galleries = download_attachments(output_path, urls)
 
 def main():
     parser = argparse.ArgumentParser(
-        description="Transform feed, Wordpress or Dotclear files to reST (rst) "
-                    "or Markdown (md) files. Be sure to have pandoc installed.",
+        description="Transform feed, WordPress, Tumblr, Dotclear, or Posterous "
+                    "files into reST (rst) or Markdown (md) files. Be sure to "
+                    "have pandoc installed.",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter)
 
     parser.add_argument(dest='input', help='The input file to read')
@@ -578,6 +729,19 @@ def main():
     parser.add_argument('--strip-raw', action='store_true', dest='strip_raw',
         help="Strip raw HTML code that can't be converted to "
              "markup such as flash embeds or iframes (wordpress import only)")
+    parser.add_argument('--wp-custpost', action='store_true',
+        dest='wp_custpost',
+        help='Put wordpress custom post types in directories. If used with '
+             '--dir-cat option directories will be created as '
+             '/post_type/category/ (wordpress import only)')
+    parser.add_argument('--wp-attach', action='store_true', dest='wp_attach',
+        help='(wordpress import only) Download files uploaded to wordpress as '
+             'attachments. Files will be added to posts as a list in the post '
+             'header. All files will be downloaded, even if '
+             "they aren't associated with a post. Files with be downloaded "
+             'with their original path inside the output directory. '
+             'e.g. output/wp-uploads/date/postname/file.jpg '
+             '-- Requires an internet connection --')
     parser.add_argument('--disable-slugs', action='store_true',
         dest='disable_slugs',
         help='Disable storing slugs from imported posts within output. '
@@ -614,8 +778,12 @@ def main():
             error = "Unable to create the output folder: " + args.output
             exit(error)
 
+    if args.wp_attach and input_type != 'wordpress':
+        error = "You must be importing a wordpress xml to use the --wp-attach option"
+        exit(error)
+
     if input_type == 'wordpress':
-        fields = wp2fields(args.input)
+        fields = wp2fields(args.input, args.wp_custpost or False)
     elif input_type == 'dotclear':
         fields = dc2fields(args.input)
     elif input_type == 'posterous':
@@ -625,6 +793,11 @@ def main():
     elif input_type == 'feed':
         fields = feed2fields(args.input)
 
+    if args.wp_attach:
+        attachments = get_attachments(args.input)
+    else:
+        attachments = None
+
     init() # init logging
 
     fields2pelican(fields, args.markup, args.output,
@@ -632,4 +805,7 @@ def main():
                    dirpage=args.dirpage or False,
                    strip_raw=args.strip_raw or False,
                    disable_slugs=args.disable_slugs or False,
-                   filter_author=args.author)
+                   filter_author=args.author,
+                   wp_custpost = args.wp_custpost or False,
+                   wp_attach = args.wp_attach or False,
+                   attachments = attachments or None)
