@@ -15,6 +15,11 @@ from pelican import __version__
 _TEMPLATES_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                               "templates")
 
+_GITHUB_PAGES_BRANCHES = {
+    'personal': 'master',
+    'project': 'gh-pages'
+}
+
 CONF = {
     'pelican': 'pelican',
     'pelicanopts': '',
@@ -31,6 +36,7 @@ CONF = {
     'cloudfiles_api_key': 'my_rackspace_api_key',
     'cloudfiles_container': 'my_cloudfiles_container',
     'dropbox_dir': '~/Dropbox/Public/',
+    'github_pages_branch': _GITHUB_PAGES_BRANCHES['project'],
     'default_pagination': 10,
     'siteurl': '',
     'lang': 'en'
@@ -180,7 +186,8 @@ needed by Pelican.
         print('Using project associated with current virtual environment.'
               'Will save to:\n%s\n' % CONF['basedir'])
     else:
-        CONF['basedir'] = os.path.abspath(ask('Where do you want to create your new web site?', answer=str_compat, default=args.path))
+        CONF['basedir'] = os.path.abspath(os.path.expanduser(
+            ask('Where do you want to create your new web site?', answer=str_compat, default=args.path)))
 
     CONF['sitename'] = ask('What will be the title of this web site?', answer=str_compat, default=args.title)
     CONF['author'] = ask('Who will be the author of this web site?', answer=str_compat, default=args.author)
@@ -217,6 +224,11 @@ needed by Pelican.
             CONF['cloudfiles_username'] = ask('What is your Rackspace Cloud username?', str_compat, CONF['cloudfiles_username'])
             CONF['cloudfiles_api_key'] = ask('What is your Rackspace Cloud API key?', str_compat, CONF['cloudfiles_api_key'])
             CONF['cloudfiles_container'] = ask('What is the name of your Cloud Files container?', str_compat, CONF['cloudfiles_container'])
+        if ask('Do you want to upload your website using GitHub Pages?', answer=bool, default=False):
+            if ask('Is this your personal page (username.github.io)?', answer=bool, default=False):
+                CONF['github_pages_branch'] = _GITHUB_PAGES_BRANCHES['personal']
+            else:
+                CONF['github_pages_branch'] = _GITHUB_PAGES_BRANCHES['project']
 
     try:
         os.makedirs(os.path.join(CONF['basedir'], 'content'))
@@ -262,9 +274,9 @@ needed by Pelican.
         try:
             with codecs.open(os.path.join(CONF['basedir'], 'Makefile'), 'w', 'utf-8') as fd:
                 mkfile_template_name = 'Makefile'
-                py_v = 'PY=python'
+                py_v = 'PY?=python'
                 if six.PY3:
-                    py_v = 'PY=python3'
+                    py_v = 'PY?=python3'
                 template = string.Template(py_v)
                 fd.write(template.safe_substitute(CONF))
                 fd.write('\n')
@@ -284,9 +296,9 @@ needed by Pelican.
         try:
             with codecs.open(os.path.join(CONF['basedir'], 'develop_server.sh'), 'w', 'utf-8') as fd:
                 lines = list(get_template('develop_server.sh'))
-                py_v = 'PY=python\n'
+                py_v = 'PY=${PY:-python}\n'
                 if six.PY3:
-                    py_v = 'PY=python3\n'
+                    py_v = 'PY=${PY:-python3}\n'
                 lines = lines[:4] + [py_v] + lines[4:]
                 for line in lines:
                     template = string.Template(line)
